let PluseIDCount = 0;

class PulseLine {
    constructor() {
        // 维护这部分的 id
        PluseIDCount++;
        this.pluseId = PluseIDCount;
    }

    loadData(pathString) {
        this.path = pathString
    }

    /**
     * @param {PulseLineSvgConfig} config 
     * @returns 
     */
    svgDoc(config) {
        config = config || {};
        const pluseStartColor = config.pluseStartColor || 'white';
        const pluseMiddleColor = config.pluseMiddleColor || '#CB81DA';
        const pluseEndColor = config.pluseEndColor || 'rgb(70, 70, 222)';
        const transform = config.transform || 'translate(0 0)';
        const duration = config.duration || 5000;

        this.duration = duration;

        const gId = 'pluse-g' + this.pluseId;
        const pId = 'pluse-p' + this.pluseId;
        const mId = 'pluse-m' + this.pluseId;
        const uId = 'pluse-u' + this.pluseId;
        const pathString = this.path;

        const parser = new DOMParser();

        const svgString = `<svg viewBox="0 0 300 100" xmlns="http://www.w3.org/2000/svg">
    <defs>
        <radialGradient id="${gId}">
        <stop offset="0%" stop-color="${pluseStartColor}" />
        <stop offset="25%" stop-color="${pluseMiddleColor}" />
        <stop offset="95%" stop-opacity="0" stop-color="${pluseEndColor}" />
        </radialGradient>
        <mask id="${mId}">
        <use id="${uId}" href="#${pId}" 
            stroke="white" stroke-width="5" fill="none"
            stroke-dasharray="100 200" stroke-linecap="round" />
        </mask>
        <path id="${pId}" d="${pathString}" />
    </defs>
    <use href="#${pId}" transform="${transform}" stroke="#CB81DA"
        stroke-width="1" fill="rgba(203, 129, 218, 0.1)"/>
    <g transform="${transform}" mask="url(#${mId})">
        <circle r="50" fill="url(#${gId})">
        <animateMotion dur="5000ms" repeatCount="indefinite">
            <mpath href="#${pId}" />
        </animateMotion>
        </circle>
    </g>
</svg>`;

        const svgDoc = parser.parseFromString(svgString, 'image/svg+xml');
        return svgDoc;
    }

    animation() {
        const pId = 'pluse-p' + this.pluseId;
        const uId = 'pluse-u' + this.pluseId;

        const pathElement = document.getElementById(pId);
        const pluseElement = document.getElementById(uId);

        const pathLength = pathElement.getTotalLength();

        const keyframes = new KeyframeEffect(
            pluseElement, [{
                strokeDasharray: `100 ${pathLength}`,
                strokeDashoffset: "100"
            },
            {
                strokeDasharray: `100 ${pathLength}`,
                strokeDashoffset: `-${pathLength - 100}`
            }
        ], {
            duration: this.duration,
            iterations: Infinity
        }
        );

        const pulsationAnimation = new Animation(keyframes, document.timeline);
        return pulsationAnimation;
    }
}

/**
 * @typedef PulseLineSvgConfig
 * @property {string} pluseStartColor
 * @property {string} pluseMiddleColor
 * @property {string} pluseEndColor
 * @property {string} transform
 * @property {number} duration
 */